﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Collections;
using System.Windows.Threading;
using System.Windows;
using System.Collections.ObjectModel;

namespace XamDataGridPerformance
{
    /// <summary>
    /// A dataset representing real-time data
    /// </summary>
    public class TickerBuffer : INotifyCollectionChanged, IEnumerable
    {
        public int WindowSize
        {
            get
            {
                return _windowSize;
            }
            set
            {
                _windowSize = value;
            }
        }

        DispatcherTimer dt = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(1000) };
        private IEnumerable _inputSource;
        private int _position = 0;
        private ObservableCollection<Row> _historicalDataSource = new ObservableCollection<Row>();
        private int _windowSize;

        public TickerBuffer() : this(DataLoader.HistoricalData, 50)
        {

        }

        /// <summary>
        /// Create a buffer using the data supplied, setting a buffer size of 500 items
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        public TickerBuffer(IEnumerable inputSource)
        {
            _inputSource = inputSource;
            _windowSize = 50;
        }

        /// <summary>
        /// Creates a buffer using the data supplied, and sets the buffer size to the specified value
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        /// <param name="windowSize">A value to be used for buffer size</param>
        public TickerBuffer(IEnumerable inputSource, int windowSize)
        {
            _inputSource = inputSource;
            _windowSize = windowSize;
            FillWindow();
            dt.Tick += new EventHandler(dt_Tick);
            dt.Start();
        }

        private static Random rnd = new Random();

        void dt_Tick(object sender, EventArgs e)
        {
            foreach (Row r in _inputSource)
            {
                r.Column1 += rnd.Next(2);
                r.Column2 += rnd.Next(2);
                r.Column3 += rnd.Next(2);
                r.Column4 += rnd.Next(2);
                r.Column5 += rnd.Next(2);
                r.Column6 += rnd.Next(2);
                r.GroupMe = rnd.Next(4).ToString();
            }
        }

        public void FillWindow()
        {
            if(_historicalDataSource.Count != _windowSize)
                this.AddRecordsFromHistoricalData(this.WindowSize);
        }

        /// <summary>
        /// Add a specified number of records containing randomly generated data to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void Generate(int recordCount)
        {
            Random r = new Random();
            DateTime startDate = new DateTime(1990, 1, 1, 0, 0, 0);

            for (int i = 0; i < recordCount; ++i)
            {
                double baseValue = r.Next(100);
                double offsetValue = r.Next(5);
                _historicalDataSource.Add(new Row(startDate.AddSeconds(1).ToShortDateString(),baseValue, baseValue + offsetValue, baseValue - offsetValue, baseValue + 2*offsetValue, 0));
            }

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from historical dataset to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount)
        {
            _historicalDataSource.Clear();

            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);

                ++_position;
            }

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from a historical dataset to this dataset, starting from a specified position
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount, int position)
        {
            _position = position;
            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);
                ++_position;
            }

            //remove old records from the beginning of the window according to window size
            while (_historicalDataSource.Count > _windowSize)
                _historicalDataSource.RemoveAt(0);

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }



        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return _historicalDataSource.GetEnumerator();
        }

        #endregion

        #region INotifyCollectionChanged Members

        public event NotifyCollectionChangedEventHandler CollectionChanged;

        #endregion
    }

    /// <summary>
    /// Row data schema
    /// </summary>
    public class Row : INotifyPropertyChanged
    {
        private static int rowID = 0;
        private static Random r = new Random();
        public Row(string date, double open, double high, double low, double close, double volume)
        {
            Date = DateTime.ParseExact(date, "M/d/yyyy", System.Globalization.CultureInfo.InvariantCulture);
            column1 = open;

        }

        private int _rowID = rowID++;

        public int RowID
        {
            get { return _rowID; }
        }

        private string _groupMe = r.Next(4).ToString();
        public string GroupMe { get { return _groupMe; } set { _groupMe = value; RaisePropertyChanged("GroupMe"); } }

        public DateTime Date { get { return column0; } set { column0 = value; RaisePropertyChanged("Date"); } }
        private DateTime column0;

        private double column1;
        public double Column1 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column1"); } }

        public double Column2 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column2"); } }

        public double Column3 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column3"); } }

        public double Column4 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column4"); } }

        public double Column5 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column5"); } }

        public double Column6 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column6"); } }

        public double Column7 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column7"); } }

        public double Column8 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column8"); } }

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
